#!/usr/bin/env python
import ConfigParser


class APCoordinatorConfig:
    # Containers
    constants = None
    config = None
    temp_config = None
    logger = None
    utilities = None

    # Constructor
    def __init__(self, ap_constants):
        self.constants = ap_constants

    # Setup the AP Utilities
    def setup_utilities(self, utilities):
        self.utilities = utilities

    # Setup the AP Logger
    def setup_logger(self, logger):
        self.logger = logger

    # Get configuration variables from the AP coordinator configuration file
    # This will favor a temporary configuration value over an actual configuration value
    # This allows for the ability to override configuration values during runtime and as options
    def get_config_var(self, config_var):
        config = self.get_config()
        temp_config = self.get_temp_config()
        config_value = None
        temp_config_value = None
        try:
            config_value = config.get('insights-coordinator-ap', config_var)
        except(ConfigParser.NoSectionError, ConfigParser.NoOptionError, ValueError, AttributeError):
            pass
        try:
            temp_config_value = temp_config.get('insights-coordinator-ap', config_var)
        except(ConfigParser.NoSectionError, ConfigParser.NoOptionError, ValueError, AttributeError):
            pass
        if config_value or temp_config_value:
            if temp_config_value != config_value:
                return temp_config_value
            return config_value
        else:
            if not hasattr(self.constants, config_var):
                return None
            return getattr(self.constants, config_var)

    # Get the entire configuration object
    def get_config(self):
        if not self.config:
            self.setup_config()
        return self.config

    # Get a temporary configuration
    # This is a copy of the current configuration but allows
    # the user to override configuration values for this one execution
    # this allows an easier method to pass in temporary
    # options that will override existing configuration values
    def get_temp_config(self):
        if not self.temp_config:
            self.setup_config()
        return self.temp_config

    # Setup the configuration object
    # This will read the configuration file and convert it
    def setup_config(self):
        self.config = ConfigParser.RawConfigParser()
        self.temp_config = ConfigParser.RawConfigParser()
        self.config.read(self.constants.conf_file)
        self.temp_config.read(self.constants.conf_file)
        if not self.config.has_section('insights-coordinator-ap'):
            self.config.add_section('insights-coordinator-ap')
        if not self.temp_config.has_section('insights-coordinator-ap'):
            self.temp_config.add_section('insights-coordinator-ap')

    # Write a single configuration variable to the configuration file
    def write_config_var(self, config_var, config_val):
        config = self.get_config()
        config.set('insights-coordinator-ap', config_var, config_val)
        with open(self.constants.conf_file, 'wb') as configfile:
            config.write(configfile)
        configfile.close()

    # Write the entire configuration file
    # { key1:val1, key2:val2 }
    def write_config_file(self, configuration_values):
        config = self.get_config()
        for key, value in configuration_values.iteritems():
            config.set('insights-coordinator-ap', key, value)
        with open(self.constants.conf_file, 'wb') as configfile:
            config.write(configfile)
        configfile.close()

    def set_temp_config_var(self, temp_config_var, temp_config_val):
        temp_config = self.get_temp_config()
        temp_config.set('insights-coordinator-ap', temp_config_var, temp_config_val)

    # Run the configuration utility
    # This will prompt for various user input for configuration variables then
    # Will rewrite the configuration file
    def run_config(self):

        self.logger.vlog("Running the AP Insights Coordinator Configuration Utility...")

        # TODO: This stuff should be in a language file
        # TODO: Create language.py -> self.langage.get_language('api_url') == 'Please enter the URL/Host...'

        # Configuration container
        config_values = {}

        ############
        # Prompt User for Configuration information
        ####

        # TODO: Delete this!
        # # API Configuration Credentials
        # ####
        # # Default API Options to be collected
        # config_values['api_options'] = self.constants.api_options
        # # URL/Host
        # config_values['api_url'] = raw_input(
        #     'Please enter the URL/Host for the RHEV Manager API. (Press enter for default %s): ' % (
        #         self.constants.api_url)) or self.constants.api_url
        # # Username
        # config_values['api_username'] = raw_input(
        #     'Please enter the username to use for the RHEV Manager API, please note that this user must have Adminstrative capabilities. (Press enter for default %s): ' % (
        #         self.constants.api_username)) or self.constants.api_username
        # # Password
        # config_values['api_password'] = getpass.getpass(
        #     'Please enter the password to use for the RHEV Manager API User %s. (Press Enter for the default %s): ' % (
        #     config_values['api_username'], self.constants.api_password)) or self.constants.api_password
        #
        # # SSH Configuration Credentials
        # ####
        # # User
        # config_values['ssh_user'] = raw_input(
        #     'Please enter the default SSH User to use for Hypervisors. (Press Enter for default %s): ' % (
        #         self.constants.ssh_user)) or self.constants.ssh_user
        # # Pass
        # config_values['ssh_pass'] = getpass.getpass(
        #     'Please enter the default SSH Password to use for Hypervisors. (Press Enter for default %s): ' % (
        #         self.constants.ssh_pass)) or self.constants.ssh_pass
        # # Port
        # config_values['ssh_port'] = raw_input(
        #     'Please enter the default SSH Port to use for Hypervisors. (Press Enter for default %s): ' % (
        #         self.constants.ssh_port)) or self.constants.ssh_port
        # # Key
        # config_values['ssh_key'] = raw_input(
        #     'Please enter the default SSH Key to use for Hypervisors. (Press Enter for default %s): ' % (
        #         self.constants.ssh_key)) or self.constants.ssh_key
        #
        # Cron Configuration
        ####
        # Would you like the cron to run daily?
        config_values['cron_symlink'] = "/etc/cron.daily/insights-coordinator-ap"
        cron_string = "Would you like to enable the RHEV Insights Coordinator to run daily? Yy/Nn: "
        yes_values = ['y', 'yes', 'yea', 'yeah', 'yep', 'mmmhmm']
        no_values = ['n', 'no', 'nope', 'naw', 'nah']
        enable_cron = raw_input(cron_string).lower().strip().rstrip()
        while (enable_cron not in yes_values) and (enable_cron not in no_values):
            enable_cron = raw_input(cron_string).lower().strip().rstrip()
        if enable_cron in yes_values:
            config_values['enable_cron'] = True
        else:
            config_values['enable_cron'] = False

        ##################
        # Default Values
        ####
        config_values['verbose'] = False
        config_values['log_backup_count'] = 3
        config_values['insights_upload'] = True
        config_values['archive_only'] = False
        # This one will go away once we have the RPMs installed on the Hypervisor Images, but for now...
        config_values['local_rhevm_installation'] = "/var/lib/redhat_access_insights/redhat-access-insights-*.tar.gz"

        # Write all user configuration values to the file
        self.logger.vlog(
            "Finished running the RHEV Insights Coordinator Configuration Utility. Writing configuration to %s." % (
                self.constants.conf_file))
        self.write_config_file(config_values)
        self.setup_config()
